'use strict';

var AST8 = require('@eslint-react/ast');
var eff = require('@eslint-react/eff');
var JSX4 = require('@eslint-react/jsx');
var types = require('@typescript-eslint/types');
var VAR = require('@eslint-react/var');
var shared = require('@eslint-react/shared');

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var AST8__namespace = /*#__PURE__*/_interopNamespace(AST8);
var JSX4__namespace = /*#__PURE__*/_interopNamespace(JSX4);
var VAR__namespace = /*#__PURE__*/_interopNamespace(VAR);

// src/component/component-collector.ts
function getElementNameAndRepresentName(node, context, polymorphicPropName, additionalComponents = []) {
  const name = JSX4__namespace.getElementName(node);
  if (name === name.toLowerCase()) return [name, name];
  const component = additionalComponents.findLast((c) => c.name === name || c.re.test(name));
  if (component != null) return [name, component.as];
  if (polymorphicPropName == null) return [name, name];
  const initialScope = context.sourceCode.getScope(node);
  const polymorphicProp = JSX4__namespace.findPropInAttributes(
    polymorphicPropName,
    initialScope,
    node.attributes
  );
  if (polymorphicProp == null) return [name, name];
  const polymorphicName = VAR__namespace.toResolved(JSX4__namespace.getPropValue(polymorphicProp, initialScope)).value;
  if (typeof polymorphicName === "string") {
    return [name, polymorphicName];
  }
  return [name, name];
}

// src/utils/get-id.ts
var id = 0n;
var getId = () => (id++).toString();
function isInitializedFromReact(name, initialScope, importSource = "react") {
  return name.toLowerCase() === "react" || VAR__namespace.isInitializedFromSource(
    name,
    importSource,
    initialScope
  );
}

// src/utils/is-from-react.ts
function isFromReact(name) {
  return (node, context) => {
    const settings = shared.unsafeDecodeSettings(context.settings);
    if (!settings.strictImportCheck) {
      if (node.type === types.AST_NODE_TYPES.MemberExpression) {
        return node.object.type === types.AST_NODE_TYPES.Identifier && node.property.type === types.AST_NODE_TYPES.Identifier && node.property.name === name;
      }
      return node.name === name;
    }
    const initialScope = context.sourceCode.getScope(node);
    if (node.type === types.AST_NODE_TYPES.MemberExpression) {
      return node.object.type === types.AST_NODE_TYPES.Identifier && node.property.type === types.AST_NODE_TYPES.Identifier && node.property.name === name && isInitializedFromReact(node.object.name, initialScope, settings.importSource);
    }
    if (node.name === name) {
      return isInitializedFromReact(name, initialScope, settings.importSource);
    }
    return false;
  };
}
function isFromReactMember(memberName, name) {
  return (node, context) => {
    const settings = shared.unsafeDecodeSettings(context.settings);
    if (!settings.strictImportCheck) {
      if (node.property.type !== types.AST_NODE_TYPES.Identifier || node.property.name !== name) {
        return false;
      }
      if (node.object.type === types.AST_NODE_TYPES.Identifier && node.object.name === memberName) {
        return true;
      }
      if (node.object.type === types.AST_NODE_TYPES.MemberExpression && node.object.object.type === types.AST_NODE_TYPES.Identifier && node.object.property.type === types.AST_NODE_TYPES.Identifier) {
        return node.object.property.name === memberName;
      }
      return false;
    }
    const initialScope = context.sourceCode.getScope(node);
    if (node.property.type !== types.AST_NODE_TYPES.Identifier || node.property.name !== name) {
      return false;
    }
    if (node.object.type === types.AST_NODE_TYPES.Identifier && node.object.name === memberName) {
      return isInitializedFromReact(node.object.name, initialScope, settings.importSource);
    }
    if (node.object.type === types.AST_NODE_TYPES.MemberExpression && node.object.object.type === types.AST_NODE_TYPES.Identifier && isInitializedFromReact(node.object.object.name, initialScope, settings.importSource) && node.object.property.type === types.AST_NODE_TYPES.Identifier) {
      return node.object.property.name === memberName;
    }
    return false;
  };
}
function isCallFromReact(name) {
  return eff.dual(2, (node, context) => {
    if (node.type !== types.AST_NODE_TYPES.CallExpression) {
      return false;
    }
    if (!AST8__namespace.isOneOf([types.AST_NODE_TYPES.Identifier, types.AST_NODE_TYPES.MemberExpression])(node.callee)) {
      return false;
    }
    return isFromReact(name)(node.callee, context);
  });
}
function isCallFromReactMember(pragmaMemberName, name) {
  return eff.dual(2, (node, context) => {
    if (node.type !== types.AST_NODE_TYPES.CallExpression) {
      return false;
    }
    if (!AST8__namespace.is(types.AST_NODE_TYPES.MemberExpression)(node.callee)) {
      return false;
    }
    return isFromReactMember(pragmaMemberName, name)(node.callee, context);
  });
}

// src/utils/is-react-api.ts
function isReactAPIWithName(name, member) {
  return member != null ? isFromReactMember(name, member) : isFromReact(name);
}
function isReactAPICallWithName(name, member) {
  return member != null ? isCallFromReactMember(name, member) : isCallFromReact(name);
}
var isChildrenCount = isReactAPIWithName("Children", "count");
var isChildrenForEach = isReactAPIWithName("Children", "forEach");
var isChildrenMap = isReactAPIWithName("Children", "map");
var isChildrenOnly = isReactAPIWithName("Children", "only");
var isChildrenToArray = isReactAPIWithName("Children", "toArray");
var isCloneElement = isReactAPIWithName("cloneElement");
var isCreateContext = isReactAPIWithName("createContext");
var isCreateElement = isReactAPIWithName("createElement");
var isCreateRef = isReactAPIWithName("createRef");
var isForwardRef = isReactAPIWithName("forwardRef");
var isMemo = isReactAPIWithName("memo");
var isChildrenCountCall = isReactAPICallWithName("Children", "count");
var isChildrenForEachCall = isReactAPICallWithName("Children", "forEach");
var isChildrenMapCall = isReactAPICallWithName("Children", "map");
var isChildrenOnlyCall = isReactAPICallWithName("Children", "only");
var isChildrenToArrayCall = isReactAPICallWithName("Children", "toArray");
var isCloneElementCall = isReactAPICallWithName("cloneElement");
var isCreateContextCall = isReactAPICallWithName("createContext");
var isCreateElementCall = isReactAPICallWithName("createElement");
var isCreateRefCall = isReactAPICallWithName("createRef");
var isForwardRefCall = isReactAPICallWithName("forwardRef");
var isMemoCall = isReactAPICallWithName("memo");

// src/element/hierarchy.ts
function isInsideCreateElementProps(node, context) {
  const call = AST8__namespace.findParentNode(node, isCreateElementCall(context));
  if (call == null) return false;
  const prop = AST8__namespace.findParentNode(node, AST8__namespace.is(types.AST_NODE_TYPES.ObjectExpression));
  if (prop == null) return false;
  return prop === call.arguments[1];
}
function isChildrenOfCreateElement(node, context) {
  const parent = node.parent;
  if (parent == null || parent.type !== types.AST_NODE_TYPES.CallExpression) return false;
  if (!isCreateElementCall(parent, context)) return false;
  return parent.arguments.slice(2).some((arg) => arg === node);
}

// src/hook/hook-name.ts
var RE_HOOK_NAME = /^use[A-Z\d]/u;
function isReactHookName(name) {
  return name === "use" || RE_HOOK_NAME.test(name);
}

// src/hook/is.ts
function isReactHook(node) {
  if (node == null) return eff._;
  const id2 = AST8__namespace.getFunctionIdentifier(node);
  return id2?.name != null && isReactHookName(id2.name);
}
function isReactHookCall(node) {
  if (node == null) return false;
  if (node.type !== types.AST_NODE_TYPES.CallExpression) {
    return false;
  }
  if (node.callee.type === types.AST_NODE_TYPES.Identifier) {
    return isReactHookName(node.callee.name);
  }
  if (node.callee.type === types.AST_NODE_TYPES.MemberExpression) {
    return node.callee.property.type === types.AST_NODE_TYPES.Identifier && isReactHookName(node.callee.property.name);
  }
  return false;
}
function isReactHookCallWithName(node, context) {
  if (node == null) return eff.returnFalse;
  const settings = shared.unsafeDecodeSettings(context.settings);
  return (name) => {
    const initialScope = context.sourceCode.getScope(node);
    switch (true) {
      case (node.callee.type === types.AST_NODE_TYPES.Identifier && node.callee.name === name):
        return !settings.strictImportCheck || isInitializedFromReact(name, initialScope, settings.importSource);
      case (node.callee.type === types.AST_NODE_TYPES.MemberExpression && node.callee.property.type === types.AST_NODE_TYPES.Identifier && node.callee.property.name === name && "name" in node.callee.object):
        return !settings.strictImportCheck || isInitializedFromReact(node.callee.object.name, initialScope, settings.importSource);
      default:
        return false;
    }
  };
}
function isReactHookCallWithNameLoose(node) {
  if (node == null) return eff.returnFalse;
  return (name) => {
    switch (node.callee.type) {
      case types.AST_NODE_TYPES.Identifier:
        return node.callee.name === name;
      case types.AST_NODE_TYPES.MemberExpression:
        return node.callee.property.type === types.AST_NODE_TYPES.Identifier && node.callee.property.name === name;
      default:
        return false;
    }
  };
}
function isReactHookCallWithNameAlias(name, context, alias) {
  const settings = shared.unsafeDecodeSettings(context);
  return (node) => {
    const initialScope = context.sourceCode.getScope(node);
    switch (true) {
      case (node.callee.type === types.AST_NODE_TYPES.Identifier && node.callee.name === name):
        return !settings.strictImportCheck || isInitializedFromReact(name, initialScope, settings.importSource);
      case (node.callee.type === types.AST_NODE_TYPES.MemberExpression && node.callee.property.type === types.AST_NODE_TYPES.Identifier && node.callee.property.name === name && "name" in node.callee.object):
        return !settings.strictImportCheck || isInitializedFromReact(node.callee.object.name, initialScope, settings.importSource);
      default:
        return alias.some(isReactHookCallWithNameLoose(node));
    }
  };
}
function isUseEffectCallLoose(node) {
  if (node == null) return false;
  if (node.type !== types.AST_NODE_TYPES.CallExpression) {
    return false;
  }
  switch (node.callee.type) {
    case types.AST_NODE_TYPES.Identifier:
      return /^use\w*Effect$/u.test(node.callee.name);
    case types.AST_NODE_TYPES.MemberExpression:
      return node.callee.property.type === types.AST_NODE_TYPES.Identifier && /^use\w*Effect$/u.test(node.callee.property.name);
    default:
      return false;
  }
}
var isUseCallbackCall = eff.flip(isReactHookCallWithName)("useCallback");
var isUseContextCall = eff.flip(isReactHookCallWithName)("useContext");
var isUseDebugValueCall = eff.flip(isReactHookCallWithName)("useDebugValue");
var isUseDeferredValueCall = eff.flip(isReactHookCallWithName)("useDeferredValue");
var isUseEffectCall = eff.flip(isReactHookCallWithName)("useEffect");
var isUseIdCall = eff.flip(isReactHookCallWithName)("useId");
var isUseImperativeHandleCall = eff.flip(isReactHookCallWithName)("useImperativeHandle");
var isUseInsertionEffectCall = eff.flip(isReactHookCallWithName)("useInsertionEffect");
var isUseLayoutEffectCall = eff.flip(isReactHookCallWithName)("useLayoutEffect");
var isUseMemoCall = eff.flip(isReactHookCallWithName)("useMemo");
var isUseReducerCall = eff.flip(isReactHookCallWithName)("useReducer");
var isUseRefCall = eff.flip(isReactHookCallWithName)("useRef");
var isUseStateCall = eff.flip(isReactHookCallWithName)("useState");
var isUseSyncExternalStoreCall = eff.flip(isReactHookCallWithName)("useSyncExternalStore");
var isUseTransitionCall = eff.flip(isReactHookCallWithName)("useTransition");

// src/hook/hierarchy.ts
function isInsideReactHook(node) {
  return isReactHook(AST8__namespace.findParentNode(node, AST8__namespace.isFunction));
}
function useHookCollector() {
  const hooks = /* @__PURE__ */ new Map();
  const fEntries = [];
  const onFunctionEnter = (node) => {
    const id2 = AST8__namespace.getFunctionIdentifier(node);
    const name = id2?.name;
    if (name != null && isReactHookName(name)) {
      const key = getId();
      fEntries.push({ key, node });
      hooks.set(key, {
        id: id2,
        key,
        kind: "function",
        name,
        node,
        flag: 0n,
        hint: 0n,
        hookCalls: []
      });
      return;
    }
    fEntries.push({ key: eff._, node });
  };
  const onFunctionExit = () => {
    fEntries.pop();
  };
  const ctx = {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    getAllHooks(node) {
      return hooks;
    },
    getCurrentHooks() {
      return new Map(hooks);
    }
  };
  const listeners = {
    ":function[type]": onFunctionEnter,
    ":function[type]:exit": onFunctionExit,
    "CallExpression[type]"(node) {
      if (!isReactHookCall(node)) {
        return;
      }
      const fEntry = fEntries.at(-1);
      if (fEntry?.key == null) {
        return;
      }
      const hook = hooks.get(fEntry.key);
      if (hook == null) {
        return;
      }
      hook.hookCalls.push(node);
    }
  };
  return { ctx, listeners };
}
var ERComponentHint = {
  ...JSX4__namespace.JSXValueHint,
  // 1n << 0n - 1n << 63n are reserved for JSXValueHint
  // Skip function component created by React.memo
  SkipMemo: 1n << 64n,
  // Skip function component created by React.forwardRef
  SkipForwardRef: 1n << 65n,
  // Skip function component defined in map function callback
  SkipMapCallback: 1n << 66n,
  // Skip function component defined on object method
  SkipObjectMethod: 1n << 67n,
  // Skip function component defined on class method
  SkipClassMethod: 1n << 68n,
  // Skip function component defined on class property
  SkipClassProperty: 1n << 69n
};
var DEFAULT_COMPONENT_HINT = 0n | ERComponentHint.SkipBooleanLiteral | ERComponentHint.SkipMapCallback | ERComponentHint.SkipNumberLiteral | ERComponentHint.SkipStringLiteral | ERComponentHint.SkipUndefined | ERComponentHint.SkipEmptyArray | ERComponentHint.StrictArray | ERComponentHint.StrictConditional | ERComponentHint.StrictLogical;

// src/component/component-flag.ts
var ERClassComponentFlag = {
  None: 0n,
  PureComponent: 1n << 0n
  // Reserved for future use
  // CreateElement: 1n << 1n,
};
var ERFunctionComponentFlag = {
  None: 0n,
  Memo: 1n << 0n,
  ForwardRef: 1n << 1n
  // Reserved for future use
  // CreateElement: 1n << 2n,
  // Reserved for future use
  // hasHooks: 1n << 3n,
  // Reserved for future use
  // Async: 1n << 4n,
};
function isComponentWrapperCall(node, context) {
  if (node.type !== types.AST_NODE_TYPES.CallExpression) {
    return false;
  }
  return isMemoCall(node, context) || isForwardRefCall(node, context) || isReactHookCallWithNameLoose(node)("useCallback");
}
function getFunctionComponentIdentifier(node, context) {
  const functionId = AST8__namespace.getFunctionIdentifier(node);
  if (functionId != null) {
    return functionId;
  }
  const { parent } = node;
  if (parent.type === types.AST_NODE_TYPES.CallExpression && isComponentWrapperCall(parent, context) && parent.parent.type === types.AST_NODE_TYPES.VariableDeclarator && parent.parent.id.type === types.AST_NODE_TYPES.Identifier) {
    return parent.parent.id;
  }
  if (parent.type === types.AST_NODE_TYPES.CallExpression && isComponentWrapperCall(parent, context) && parent.parent.type === types.AST_NODE_TYPES.CallExpression && isComponentWrapperCall(parent.parent, context) && parent.parent.parent.type === types.AST_NODE_TYPES.VariableDeclarator && parent.parent.parent.id.type === types.AST_NODE_TYPES.Identifier) {
    return parent.parent.parent.id;
  }
  return eff._;
}
var RE_COMPONENT_NAME = /^_?[A-Z]/u;
function getComponentNameFromIdentifier(node) {
  if (node == null) return eff._;
  return Array.isArray(node) ? node.map((n) => n.name).join(".") : node.name;
}
function isComponentName(name) {
  return RE_COMPONENT_NAME.test(name);
}
function isClassComponent(node) {
  if ("superClass" in node && node.superClass != null) {
    const re = /^(?:Pure)?Component$/u;
    switch (true) {
      case node.superClass.type === types.AST_NODE_TYPES.Identifier:
        return re.test(node.superClass.name);
      case (node.superClass.type === types.AST_NODE_TYPES.MemberExpression && node.superClass.property.type === types.AST_NODE_TYPES.Identifier):
        return re.test(node.superClass.property.name);
    }
  }
  return false;
}
function isPureComponent(node) {
  if ("superClass" in node && node.superClass != null) {
    const re = /^PureComponent$/u;
    switch (true) {
      case node.superClass.type === types.AST_NODE_TYPES.Identifier:
        return re.test(node.superClass.name);
      case (node.superClass.type === types.AST_NODE_TYPES.MemberExpression && node.superClass.property.type === types.AST_NODE_TYPES.Identifier):
        return re.test(node.superClass.property.name);
    }
  }
  return false;
}

// src/component/component-render-method.ts
function isRenderMethodLike(node) {
  return (node.type === types.AST_NODE_TYPES.MethodDefinition || node.type === types.AST_NODE_TYPES.PropertyDefinition) && node.key.type === types.AST_NODE_TYPES.Identifier && node.key.name === "render" && node.parent.parent.type === types.AST_NODE_TYPES.ClassDeclaration;
}
function isFunctionOfRenderMethod(node) {
  if (!isRenderMethodLike(node.parent)) {
    return false;
  }
  return isClassComponent(node.parent.parent.parent);
}
function isInsideRenderMethod(node) {
  return AST8__namespace.findParentNode(node, (node2) => isRenderMethodLike(node2) && isClassComponent(node2.parent.parent)) != null;
}

// src/component/misc.ts
function hasNoneOrValidComponentName(node, context) {
  const id2 = getFunctionComponentIdentifier(node, context);
  if (id2 == null) return true;
  const name = Array.isArray(id2) ? id2.at(-1)?.name : id2.name;
  return name != null && isComponentName(name);
}

// src/component/component-collector.ts
function hasValidHierarchy(node, context, hint) {
  if (isChildrenOfCreateElement(node, context) || isFunctionOfRenderMethod(node)) {
    return false;
  }
  if (hint & ERComponentHint.SkipMapCallback && AST8__namespace.isMapCallLoose(node.parent)) {
    return false;
  }
  if (hint & ERComponentHint.SkipObjectMethod && AST8__namespace.isFunctionOfObjectMethod(node.parent)) {
    return false;
  }
  if (hint & ERComponentHint.SkipClassMethod && AST8__namespace.isFunctionOfClassMethod(node.parent)) {
    return false;
  }
  if (hint & ERComponentHint.SkipClassProperty && AST8__namespace.isFunctionOfClassProperty(node.parent)) {
    return false;
  }
  const boundaryNode = AST8__namespace.findParentNode(
    node,
    AST8__namespace.isOneOf([
      types.AST_NODE_TYPES.JSXExpressionContainer,
      types.AST_NODE_TYPES.ArrowFunctionExpression,
      types.AST_NODE_TYPES.FunctionExpression,
      types.AST_NODE_TYPES.Property,
      types.AST_NODE_TYPES.ClassBody
    ])
  );
  return boundaryNode == null || boundaryNode.type !== types.AST_NODE_TYPES.JSXExpressionContainer;
}
function getComponentFlag(initPath) {
  let flag = ERFunctionComponentFlag.None;
  if (initPath != null && AST8__namespace.hasCallInFunctionInitPath("memo", initPath)) {
    flag |= ERFunctionComponentFlag.Memo;
  }
  if (initPath != null && AST8__namespace.hasCallInFunctionInitPath("forwardRef", initPath)) {
    flag |= ERFunctionComponentFlag.ForwardRef;
  }
  return flag;
}
var displayNameAssignmentSelector = "AssignmentExpression[type][operator='='][left.type='MemberExpression'][left.property.name='displayName']";
function useComponentCollector(context, hint = DEFAULT_COMPONENT_HINT, options = {}) {
  const { collectDisplayName = false, collectHookCalls = false } = options;
  const jsxCtx = { getScope: (node) => context.sourceCode.getScope(node) };
  const components = /* @__PURE__ */ new Map();
  const functionEntries = [];
  const getCurrentEntry = () => functionEntries.at(-1);
  const onFunctionEnter = (node) => {
    const key = getId();
    functionEntries.push({ key, node, hookCalls: [], isComponent: false });
  };
  const onFunctionExit = () => {
    const entry = functionEntries.at(-1);
    if (entry == null) return;
    if (!entry.isComponent) {
      return functionEntries.pop();
    }
    const shouldDrop = AST8__namespace.getNestedReturnStatements(entry.node.body).slice().reverse().some((r) => {
      return context.sourceCode.getScope(r).block === entry.node && r.argument != null && !JSX4__namespace.isJSXValue(r.argument, jsxCtx, hint);
    });
    if (shouldDrop) {
      components.delete(entry.key);
    }
    return functionEntries.pop();
  };
  const ctx = {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    getAllComponents(node) {
      return components;
    },
    getCurrentEntries() {
      return [...functionEntries];
    },
    getCurrentEntry
  };
  const listeners = {
    ":function[type]": onFunctionEnter,
    ":function[type]:exit": onFunctionExit,
    "ArrowFunctionExpression[type][body.type!='BlockStatement']"() {
      const entry = getCurrentEntry();
      if (entry == null) {
        return;
      }
      const { body } = entry.node;
      const isComponent = hasNoneOrValidComponentName(entry.node, context) && JSX4__namespace.isJSXValue(body, jsxCtx, hint) && hasValidHierarchy(entry.node, context, hint);
      if (!isComponent) {
        return;
      }
      const initPath = AST8__namespace.getFunctionInitPath(entry.node);
      const id2 = getFunctionComponentIdentifier(entry.node, context);
      const name = getComponentNameFromIdentifier(id2);
      const key = getId();
      components.set(key, {
        id: id2,
        key,
        kind: "function",
        name,
        node: entry.node,
        displayName: eff._,
        flag: getComponentFlag(initPath),
        hint,
        hookCalls: entry.hookCalls,
        initPath
      });
    },
    ...collectDisplayName ? {
      [displayNameAssignmentSelector](node) {
        const { left, right } = node;
        const componentName = left.object.type === types.AST_NODE_TYPES.Identifier ? left.object.name : eff._;
        const component = [...components.values()].findLast(({ name }) => name != null && name === componentName);
        if (component == null) {
          return;
        }
        component.displayName = right;
      }
    } : {},
    ...collectHookCalls ? {
      "CallExpression[type]:exit"(node) {
        if (!isReactHookCall(node)) {
          return;
        }
        const entry = getCurrentEntry();
        if (entry == null) {
          return;
        }
        entry.hookCalls.push(node);
      }
    } : {},
    "ReturnStatement[type]"(node) {
      const entry = getCurrentEntry();
      if (entry == null) {
        return;
      }
      const isComponent = hasNoneOrValidComponentName(entry.node, context) && JSX4__namespace.isJSXValue(node.argument, jsxCtx, hint) && hasValidHierarchy(entry.node, context, hint);
      if (!isComponent) {
        return;
      }
      functionEntries.pop();
      functionEntries.push({ ...entry, isComponent });
      const initPath = AST8__namespace.getFunctionInitPath(entry.node);
      const id2 = getFunctionComponentIdentifier(entry.node, context);
      const name = getComponentNameFromIdentifier(id2);
      components.set(entry.key, {
        id: id2,
        key: entry.key,
        kind: "function",
        name,
        node: entry.node,
        displayName: eff._,
        flag: getComponentFlag(initPath),
        hint,
        hookCalls: entry.hookCalls,
        initPath
      });
    }
  };
  return { ctx, listeners };
}
function useComponentCollectorLegacy() {
  const components = /* @__PURE__ */ new Map();
  const ctx = {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    getAllComponents(node) {
      return components;
    }
  };
  const collect = (node) => {
    if (!isClassComponent(node)) {
      return;
    }
    const id2 = AST8__namespace.getClassIdentifier(node);
    const key = getId();
    const flag = isPureComponent(node) ? ERClassComponentFlag.PureComponent : ERClassComponentFlag.None;
    components.set(
      key,
      {
        id: id2,
        key,
        kind: "class",
        name: id2?.name,
        node,
        // TODO: Get displayName of class component
        displayName: eff._,
        flag,
        hint: 0n,
        // TODO: Get methods of class component
        methods: []
      }
    );
  };
  const listeners = {
    "ClassDeclaration[type]": collect,
    "ClassExpression[type]": collect
  };
  return { ctx, listeners };
}
function isSetupFunction(node) {
  if (node == null) return eff._;
  return node.parent?.type === types.AST_NODE_TYPES.CallExpression && node.parent.callee !== node && node.parent.callee.type === types.AST_NODE_TYPES.Identifier && node.parent.arguments.at(0) === node && isUseEffectCallLoose(node.parent);
}
function isCleanupFunction(node) {
  const nearReturn = AST8__namespace.findParentNode(node, AST8__namespace.is(types.AST_NODE_TYPES.ReturnStatement));
  const nearFunction = AST8__namespace.findParentNode(node, AST8__namespace.isFunction);
  const nearFunctionOfReturn = AST8__namespace.findParentNode(nearReturn, AST8__namespace.isFunction);
  return nearFunction === nearFunctionOfReturn && isSetupFunction(nearFunction);
}
function isComponentDidMount(node) {
  return AST8__namespace.isOneOf([types.AST_NODE_TYPES.MethodDefinition, types.AST_NODE_TYPES.PropertyDefinition])(node) && node.key.type === types.AST_NODE_TYPES.Identifier && node.key.name === "componentDidMount";
}
function isComponentWillUnmount(node) {
  return AST8__namespace.isOneOf([types.AST_NODE_TYPES.MethodDefinition, types.AST_NODE_TYPES.PropertyDefinition])(node) && node.key.type === types.AST_NODE_TYPES.Identifier && node.key.name === "componentWillUnmount";
}
function isComponentDidMountFunction(node) {
  return AST8__namespace.isFunction(node) && isComponentDidMount(node.parent) && node.parent.value === node;
}
function isComponentWillUnmountFunction(node) {
  return AST8__namespace.isFunction(node) && isComponentWillUnmount(node.parent) && node.parent.value === node;
}
var ERPhaseRelevance = eff.birecord({
  mount: "unmount",
  setup: "cleanup"
});
var isInversePhase = eff.dual(2, (a, b) => ERPhaseRelevance.get(a) === b);
function isDirectValueOfRenderPropertyLoose(node) {
  const matching = (node2) => {
    return node2.type === types.AST_NODE_TYPES.Property && node2.key.type === types.AST_NODE_TYPES.Identifier && node2.key.name.startsWith("render");
  };
  return matching(node) || node.parent != null && matching(node.parent);
}
function isDeclaredInRenderPropLoose(node) {
  if (isDirectValueOfRenderPropertyLoose(node)) {
    return true;
  }
  const parent = AST8__namespace.findParentNode(node, AST8__namespace.is(types.AST_NODE_TYPES.JSXExpressionContainer))?.parent;
  if (parent?.type !== types.AST_NODE_TYPES.JSXAttribute) {
    return false;
  }
  return parent.name.type === types.AST_NODE_TYPES.JSXIdentifier && parent.name.name.startsWith("render");
}
function isRenderFunctionLoose(node, context) {
  const { body, parent } = node;
  if (AST8__namespace.getFunctionIdentifier(node)?.name.startsWith("render")) {
    return parent.type === types.AST_NODE_TYPES.JSXExpressionContainer && parent.parent.type === types.AST_NODE_TYPES.JSXAttribute && parent.parent.name.type === types.AST_NODE_TYPES.JSXIdentifier && parent.parent.name.name.startsWith("render");
  }
  return JSX4__namespace.isJSXValue(
    body,
    {
      getScope: (node2) => context.sourceCode.getScope(node2)
    },
    JSX4__namespace.JSXValueHint.SkipNullLiteral | JSX4__namespace.JSXValueHint.SkipUndefined | JSX4__namespace.JSXValueHint.StrictLogical | JSX4__namespace.JSXValueHint.StrictConditional
  );
}
function isRenderPropLoose(node, context) {
  if (node.name.type !== types.AST_NODE_TYPES.JSXIdentifier) {
    return false;
  }
  return node.name.name.startsWith("render") && node.value?.type === types.AST_NODE_TYPES.JSXExpressionContainer && AST8__namespace.isFunction(node.value.expression) && isRenderFunctionLoose(node.value.expression, context);
}
function isThisSetState(node) {
  const { callee } = node;
  return callee.type === types.AST_NODE_TYPES.MemberExpression && AST8__namespace.isThisExpression(callee.object) && callee.property.type === types.AST_NODE_TYPES.Identifier && callee.property.name === "setState";
}

exports.DEFAULT_COMPONENT_HINT = DEFAULT_COMPONENT_HINT;
exports.ERClassComponentFlag = ERClassComponentFlag;
exports.ERComponentHint = ERComponentHint;
exports.ERFunctionComponentFlag = ERFunctionComponentFlag;
exports.ERPhaseRelevance = ERPhaseRelevance;
exports.RE_COMPONENT_NAME = RE_COMPONENT_NAME;
exports.RE_HOOK_NAME = RE_HOOK_NAME;
exports.getComponentNameFromIdentifier = getComponentNameFromIdentifier;
exports.getElementNameAndRepresentName = getElementNameAndRepresentName;
exports.getFunctionComponentIdentifier = getFunctionComponentIdentifier;
exports.getId = getId;
exports.hasNoneOrValidComponentName = hasNoneOrValidComponentName;
exports.isCallFromReact = isCallFromReact;
exports.isCallFromReactMember = isCallFromReactMember;
exports.isChildrenCount = isChildrenCount;
exports.isChildrenCountCall = isChildrenCountCall;
exports.isChildrenForEach = isChildrenForEach;
exports.isChildrenForEachCall = isChildrenForEachCall;
exports.isChildrenMap = isChildrenMap;
exports.isChildrenMapCall = isChildrenMapCall;
exports.isChildrenOfCreateElement = isChildrenOfCreateElement;
exports.isChildrenOnly = isChildrenOnly;
exports.isChildrenOnlyCall = isChildrenOnlyCall;
exports.isChildrenToArray = isChildrenToArray;
exports.isChildrenToArrayCall = isChildrenToArrayCall;
exports.isClassComponent = isClassComponent;
exports.isCleanupFunction = isCleanupFunction;
exports.isCloneElement = isCloneElement;
exports.isCloneElementCall = isCloneElementCall;
exports.isComponentDidMount = isComponentDidMount;
exports.isComponentDidMountFunction = isComponentDidMountFunction;
exports.isComponentName = isComponentName;
exports.isComponentWillUnmount = isComponentWillUnmount;
exports.isComponentWillUnmountFunction = isComponentWillUnmountFunction;
exports.isCreateContext = isCreateContext;
exports.isCreateContextCall = isCreateContextCall;
exports.isCreateElement = isCreateElement;
exports.isCreateElementCall = isCreateElementCall;
exports.isCreateRef = isCreateRef;
exports.isCreateRefCall = isCreateRefCall;
exports.isDeclaredInRenderPropLoose = isDeclaredInRenderPropLoose;
exports.isDirectValueOfRenderPropertyLoose = isDirectValueOfRenderPropertyLoose;
exports.isForwardRef = isForwardRef;
exports.isForwardRefCall = isForwardRefCall;
exports.isFromReact = isFromReact;
exports.isFromReactMember = isFromReactMember;
exports.isFunctionOfRenderMethod = isFunctionOfRenderMethod;
exports.isInitializedFromReact = isInitializedFromReact;
exports.isInsideCreateElementProps = isInsideCreateElementProps;
exports.isInsideReactHook = isInsideReactHook;
exports.isInsideRenderMethod = isInsideRenderMethod;
exports.isInversePhase = isInversePhase;
exports.isMemo = isMemo;
exports.isMemoCall = isMemoCall;
exports.isPureComponent = isPureComponent;
exports.isReactAPICallWithName = isReactAPICallWithName;
exports.isReactAPIWithName = isReactAPIWithName;
exports.isReactHook = isReactHook;
exports.isReactHookCall = isReactHookCall;
exports.isReactHookCallWithName = isReactHookCallWithName;
exports.isReactHookCallWithNameAlias = isReactHookCallWithNameAlias;
exports.isReactHookCallWithNameLoose = isReactHookCallWithNameLoose;
exports.isReactHookName = isReactHookName;
exports.isRenderFunctionLoose = isRenderFunctionLoose;
exports.isRenderMethodLike = isRenderMethodLike;
exports.isRenderPropLoose = isRenderPropLoose;
exports.isSetupFunction = isSetupFunction;
exports.isThisSetState = isThisSetState;
exports.isUseCallbackCall = isUseCallbackCall;
exports.isUseContextCall = isUseContextCall;
exports.isUseDebugValueCall = isUseDebugValueCall;
exports.isUseDeferredValueCall = isUseDeferredValueCall;
exports.isUseEffectCall = isUseEffectCall;
exports.isUseEffectCallLoose = isUseEffectCallLoose;
exports.isUseIdCall = isUseIdCall;
exports.isUseImperativeHandleCall = isUseImperativeHandleCall;
exports.isUseInsertionEffectCall = isUseInsertionEffectCall;
exports.isUseLayoutEffectCall = isUseLayoutEffectCall;
exports.isUseMemoCall = isUseMemoCall;
exports.isUseReducerCall = isUseReducerCall;
exports.isUseRefCall = isUseRefCall;
exports.isUseStateCall = isUseStateCall;
exports.isUseSyncExternalStoreCall = isUseSyncExternalStoreCall;
exports.isUseTransitionCall = isUseTransitionCall;
exports.useComponentCollector = useComponentCollector;
exports.useComponentCollectorLegacy = useComponentCollectorLegacy;
exports.useHookCollector = useHookCollector;
